define('oxguard/mail/maillist', [
    'io.ox/mail/common-extensions',
    'io.ox/core/extensions'
], function (extensions, ext) {
    'use strict';

    var initialized = false;

    // Override default icon extension for encrypted icons
    extensions.pgp.encrypted = function (baton) {
        if (!/^multipart\/encrypted/.test(baton.data.content_type) &&
                !(baton.model.get('security_info') && baton.model.get('security_info').encrypted) &&
                !(baton.model.get('security') && baton.model.get('security').decrypted)) return;

        this.append(
            $('<i class="fa fa-lock encrypted" aria-hidden="true">')
        );
    };

    // Changed selection.  Null out the guard current data
    function change () {
        window.oxguarddata.current = null;
    }

    // To initialize listview.  Setup here
    function listViewInit() {
        require (['io.ox/mail/main'], function (main) {
            var app = main.getApp();
            if (app.listView !== undefined) {
                app.listView.on('selection:change', change);
            }
        });
    }

    // Disable default paper clip, as all oxguard emails have attachments.  Will call manually.
    ext.point('io.ox/mail/listview/item/default/row2').disable('paper-clip');
    ext.point('io.ox/mail/listview/item/small/col4').disable('paper-clip');

    ext.point('io.ox/mail/listview/item/default/row2').extend({
        id: 'OGpaper-clip',
        index: 300,
        draw: function (baton) {
            addLock.call(this, baton);
        }
    });

    ext.point('io.ox/mail/listview/item/small/col5').disable('paper-clip');
    ext.point('io.ox/mail/listview/item/small/col5').extend({
        id: 'OGpaper-clipSm',
        index: 300,
        draw: function (baton) {
            addLock.call(this, baton);
        }
    });

    // Check if OxGuard email, and if so, add lock icon.  Add paperclip if applicable
    function addLock(baton) {
        try {
            if (!initialized) {
                listViewInit();
                initialized = true;
            }
            if (baton.data.attachment === undefined) {
                return;
            }
            if (baton.model === undefined) {
                extensions.paperClip.call(this, baton);
                return;
            }
            // Encrypted, we don't know if attachments
            if (baton.model.get('security_info') && baton.model.get('security_info').encrypted) {
                if (!(baton.model.get('security') && baton.model.get('security').decrypted)) {
                    return;
                }
            }
            if (baton.model.get('content_type') === 'multipart/encrypted') {
                return;
            }
            /// PGP Components
            // Check if decoded
            if (baton.model.get('security') && baton.model.get('security').decrypted) {
                if (baton.model.get('attachment') === true) {
                    extensions.paperClip.call(this, baton);
                } else {
                    $(this).find('.fa-paperclip').hide();  // remove if already added
                }
                return;
            }
            extensions.paperClip.call(this, baton);  // Otherwise, call paperclip
        } catch (ex) {
            console.log('Problem checking list for OG email');
            console.log(ex);
        }
    }
});
