var fs = require('fs');
var _ = require('underscore');
var localConf = {};
var seleniumProcess;

if (fs.existsSync('grunt/local.conf.json')) {
    localConf = JSON.parse(fs.readFileSync('grunt/local.conf.json')) || {};
}
localConf.e2e = localConf.e2e || {};
localConf.e2e.helpers = localConf.e2e.helpers || {};

global.Helper =

module.exports.config = {
    'tests': './e2e/tests/**/*_test.js',
    'timeout': 10000,
    'output': './build/e2e/',
    'helpers': {
        WebDriver: _.extend({}, {
            url: process.env.LAUNCH_URL || 'http://192.168.0.86:8337/appsuite/',
            host: process.env.SELENIUM_HOST || '127.0.0.1',
            smartWait: 1000,
            waitForTimeout: 10000,
            browser: 'chrome',
            restart: true,
            windowSize: 'maximize',
            desiredCapabilities: {
                browserName: 'chrome',
                chromeOptions: {
                    args: ['no-sandbox']
                }
            }
        }, localConf.e2e.helpers.WebDriver || {}),
        OpenXchange: {
            require: './node_modules/@open-xchange/codecept-helper/src/helper',
            mxDomain: localConf.e2e.mxDomain || 'example.com',
            smtpServer: localConf.e2e.smtpServer || 'localhost',
            imapServer: localConf.e2e.imapServer || 'localhost'
        },
        GuardHelper: {
            require: './e2e/helper.js'
        }
    },
    'include': {
        'I': './e2e/guardActor',
        users: './e2e/users'
    },
    'bootstrap': function (done) {
        var chai = require('chai');
        chai.config.includeStack = true;
        global.expect = chai.expect;
        global.AssertionError = chai.AssertionError;
        global.Assertion = chai.Assertion;
        global.assert = chai.assert;
        chai.Should();
        try {
            var config = require('codeceptjs').config.get();
            if (/127\.0\.0\.1/.test(config.helpers.WebDriver.host)) {
                require('@open-xchange/codecept-helper').selenium
                    .start(localConf.e2e.selenium)
                    .then(done);
            } else {
                done();
            }
        } catch (e) {
            if (e.code === 'EUSEREMOTE') return done();
            //throw again, to make the error visible
            throw e;
        }
    },
    'teardown': function () {
        //HACK: defer killing selenium, because it's still needed for a few ms
        global.setTimeout(function () {
            try {
                seleniumProcess.kill();
            } catch (e) {
                //ignore me
            }
        }, 100);
    },
    'mocha': {},
    'name': 'App Suite Core UI'
};
